/* ------------------------------------------------------------------ */
/*      item            : XmlSnapshot.hxx
        generated by    : Autobuild (abuild)
        date            : Fri 07 Mar 2025
        category        : header file
        description     : DUECA Communication Object (DCO)
                          automatically generated by dueca-codegen
        codegen version : 111
        language        : C++
        item            : XmlSnapshot.dco
        made by         : Rene' van Paassen
        date            : 1301002
        description     : XML based snapshot data
        copyright       : (c) 2013-2022 TUDelft-AE-C&S
                        : (c) 2022 Joost Ellerbroek / René van Paassen
        license         : EUPL-1.2
        documentation   : DUECA_API
*/

#pragma once
#if !defined(__DCO_NOPACK)
namespace dueca {
class AmorphStore;
class AmorphReStore;
struct DataWriterArraySize;
};
#endif

#if !defined(__DCO_STANDALONE)
namespace dueca {
struct CommObjectDataTable;
};

#include <gencodegen.h>
#if GENCODEGEN != 111
#error "Generated XmlSnapshot.hxx too old, please clean with 'make mrproper'"
#endif
#include <dueca/CommObjectTraits.hxx>
#endif
#include <iostream>

#ifndef NESTED_DCO
#define NESTED_DCO
#define MASTER_DCO_XmlSnapshot
#endif
#include <Snapshot.hxx>
#ifdef MASTER_DCO_XmlSnapshot
#undef MASTER_DCO_XmlSnapshot
#undef NESTED_DCO
#endif

namespace dueca {
 
/** \brief An XmlSnapshot can be used to communicate RTW module
    settings (inputs, states, and parameters) from and to RTW modules.
    Common scenarios are for instance loading a MATLAB generated
    initial condition for your simulink model into your RTW model,
    or retrieving the current state of your model to store in an
    xml file (for instance for later use in MATLAB).
    DUECA's new-module script gives you the option to generate
    scripts that you can use to communicate state, input and
    parameter settings between simulink and DUECA.
    If you choose to use this, new-module generates MATLAB scripts
    to export simulink model settings to an xml file, and to
    import simulink model settings from an xml file. You can choose
    to edit the generated xml files by and. If you want, you can remove
    variables from the xml file that you know you will never change.
    Loading and saving of these xml files in DUECA should be done outside
    of the actual RTW module: preferably in your 'ECI' module.
    The following example shows how to load and send an xml snapshot:
    \code
    // READ FROM FILE
    ifstream fin("myxmlsnapshot.xml");
    string line, strdata;
    while(!fin.eof()) {
      getline(fin,line);
      strdata+=line;
    }
    fin.close();
    // SEND SNAPSHOT
    XmlSnapshot *snap = new XmlSnapshot(strdata, NameSet(getEntity(), "my-rtw-module", ""));
    w_xmlsnap_example.sendEvent(snap, ts);
    \endcode
    You can also use xml snapshots to store the current state in an xml file.
    This is done in two steps: first you send a command to the RTW module to
    prepare for a snapshot, at a certain time given by you. The RTW module then
    stores its states at the chosen time, to be sent in an xml snapshot when a
    second command is given: this is when you send the command to actually send
    the snapshot. The following three examples show how to retrieve a snapshot from
    the module with these two commands, and subsequently how to handle the reply:
    \code
    // Send the command to the RTW module to prepare the snapshot
    XmlSnapshot *snap = new XmlSnapshot(XmlSnapshot::PrepareXmlSnapshot, NameSet(getEntity(), "my-rtw-module", ""));
    w_xmlsnap_example.sendEvent(snap, ts);
    \endcode
    \code
    // Send the command to the RTW module to actually send the snapshot
    XmlSnapshot *snap = new XmlSnapshot(XmlSnapshot::SendXmlSnapshot, NameSet(getEntity(), "my-rtw-module", ""));
    w_xmlsnap_example.sendEvent(snap, ts);
    \endcode
    \code
    // Act on incoming snapshots
    if (r_xmlsnap_example.getNumWaitingEvents(ts))
    {
      EventReader<XmlSnapshot> in(r_xmlsnap_example, ts);
      ofstream fout("mynewxmlsnapshot.xml");
      fout << in.data().accessData();
      fout.close();
    }
    \endcode
    */
struct XmlSnapshot : public Snapshot
{
  /** typedef for internal reference */
  typedef XmlSnapshot __ThisDCOType__;

public:
  /** The name of this class. */
  static const char* const classname;


public:
  /** a "magic" number, hashed out of the class definition,
      that will be used to check consistency of the sent objects
      across the dueca nodes. */
  static const uint32_t magic_check_number;

  /** default constructor. */
  XmlSnapshot();

  /** Constructor with arguments */
  XmlSnapshot(
        const Snapshot& parent_Snapshot);

  /** copy constructor. */
  XmlSnapshot(const XmlSnapshot& o);

#if !defined(__DCO_NOPACK)
  /** constructor to restore an XmlSnapshot from amorphous storage. */
  XmlSnapshot(::dueca::AmorphReStore& r);
#endif

  /** destructor. */
  ~XmlSnapshot();

#if !defined(__DCO_STANDALONE)
  /** new operator "new", which places objects not on a
      heap, but in one of the memory arenas. This to speed up
      memory management. */
  static void* operator new(size_t size);

  /** accompanying placement operator "new" */
  static void* operator new(size_t size, void* mem);

  /** new operator "delete", to go with the new version
      of operator new. */
  static void operator delete(void* p);

  /** placement "new", needed for stl. */
  inline static void* operator new(size_t size, XmlSnapshot*& o)
  { return reinterpret_cast<void*>(o); }
#endif

#if !defined(__DCO_NOPACK)
  /** packs the XmlSnapshot into amorphous storage. */
  void packData(::dueca::AmorphStore& s) const;

  /** packs the XmlSnapshot into amorphous storage.
      only differences with a previous object are packed. */
  void packDataDiff(::dueca::AmorphStore& s, const XmlSnapshot& ref) const;

  /** unpacks the XmlSnapshot from an amorphous storage. */
  void unPackData(::dueca::AmorphReStore& s);

  /** unpacks the differences for XmlSnapshot
      from an amorphous storage. */
  void unPackDataDiff(::dueca::AmorphReStore& s);
#endif

  /** Test for equality. */
  bool operator == (const XmlSnapshot& o) const;

  /** Test for inequality. */
  inline bool operator != (const XmlSnapshot& o) const
  { return !(*this == o); }

  /** Assignment operator. */
  XmlSnapshot& operator=(const XmlSnapshot& o);

  /** prints the XmlSnapshot to a stream. */
  std::ostream & print(std::ostream& s) const;

// Additional custom code
#include "XmlSnapshotExtra.hxx"
};

} 

#if !defined(__DCO_NOPACK)
/** pack the object into amorphous storage. */
inline void packData(::dueca::AmorphStore& s,
                     const dueca::XmlSnapshot& o)
{ o.packData(s); }

/** pack the differences between this object and another
    into amorphous storage. */
inline void packDataDiff(dueca::AmorphStore& s,
                         const dueca::XmlSnapshot& o,
                         const dueca::XmlSnapshot& ref)
{ o.packDataDiff(s, ref); }

/** unpack the object from amorphous storage. */
inline void unPackData(::dueca::AmorphReStore& s,
                       dueca::XmlSnapshot& o)
{ o.unPackData(s); }

/** unpack the differences to this object from storage. */
inline void unPackDataDiff(dueca::AmorphReStore& s,
                           dueca::XmlSnapshot& o)
{ o.unPackDataDiff(s); }
#endif

namespace std {
/** print to a stream. */
inline std::ostream &
operator << (std::ostream& s, const dueca::XmlSnapshot& o)
{ return o.print(s); }
};

// specialization of classname
template<>
const char* dueca::getclassname<dueca::XmlSnapshot>();
template <>
inline const char* dueca::getclassname<const dueca::XmlSnapshot>()
{ return dueca::getclassname<dueca::XmlSnapshot>(); }

#if !defined(__DCO_STANDALONE)
namespace dueca {
/** Template specialization, defines a trait that is needed if
    XmlSnapshot is ever used inside other dco objects. */
template <>
struct dco_nested<dueca::XmlSnapshot> : public dco_isnested { };
};

#endif


